import requests


def fetch(url, **kwargs):
    """
    fetch api response
    :param url: fetch url
    :param kwargs: other requests params
    :return: json of response
    """
    kwargs.update({'verify': False})
    response = requests.get(url, **kwargs)
    if response.status_code != 200:
        raise requests.ConnectionError('Expected status code 200, but got {}'.format(response.status_code))
    return response.json()



from douyin.structures import Music
from douyin.handlers import Handler
from douyin.downloaders import Downloader


class MusicDownloader(Downloader):
    
    async def process_item(self, obj):
        """
        process item
        :param obj: single obj
        :return:
        """
        if isinstance(obj, Music):
            print('Downloading', obj, '...')
            for handler in self.handlers:
                if isinstance(handler, Handler):
                    await handler.process(obj.play_url, obj.id)



hot_search_url = 'https://api.amemv.com/aweme/v1/hot/search/list/'
hot_video_url = 'https://api.amemv.com/aweme/v1/hotsearch/aweme/billboard/'
hot_energy_url = 'https://api.amemv.com/aweme/v1/hotsearch/positive_energy/billboard/'
hot_music_url  ='https://api.amemv.com/aweme/v1/hotsearch/music/billboard/'


from douyin.structures import Video, User, Music, Address
from douyin.utils.common import first, parse_datetime


def get_video_url(array):
    """
    get video url from list
    :param array:
    :return:
    """
    if isinstance(array, list) and len(array) >= 1:
        return array[-1]
    return None


def get_music_url(array):
    """
    get music url from list
    :param array:
    :return:
    """
    if isinstance(array, list) and len(array) >= 1:
        return array[-1]
    return None


def data_to_video(data):
    """
    transfer json data to video object
    :param data:
    :return:
    """
    statistics = data.get('statistics', {})
    like_count = statistics.get('digg_count')
    comment_count = statistics.get('comment_count')
    share_count = statistics.get('share_count')
    id = statistics.get('aweme_id')
    desc = data.get('desc')
    is_ads = data.get('is_ads')
    duration = data.get('duration')
    create_time = parse_datetime(data.get('create_time'))
    share_url = data.get('share_url')
    ratio = data.get('video', {}).get('ratio')
    cover_url = first(data.get('video', {}).get('origin_cover', {}).get('url_list'))
    play_url = get_video_url(data.get('video', {}).get('play_addr', {}).get('url_list'))
    author = data_to_user(data.get('author', {}))
    music = data_to_music(data.get('music', {}))
    address = data_to_address(data.get('poi_info', {}))
    return Video(
        id=id,
        desc=desc,
        like_count=like_count,
        share_count=share_count,
        comment_count=comment_count,
        is_ads=is_ads,
        duration=duration,
        create_time=create_time,
        share_url=share_url,
        ratio=ratio,
        cover_url=cover_url,
        play_url=play_url,
        author=author,
        music=music,
        address=address
    ) if id else None


def data_to_music(data):
    """
    transfer data to music object
    :param data:
    :return:
    """
    id = data.get('mid')
    name = data.get('title')
    play_url = get_music_url(data.get('play_url', {}).get('url_list', []))
    owner_id = data.get('owner_id')
    owner_name = data.get('owner_nickname')
    cover_url = first(data.get('cover_large', {}).get('url_list', []))
    return Music(
        id=id,
        name=name,
        play_url=play_url,
        owner_id=owner_id,
        owner_name=owner_name,
        cover_url=cover_url
    ) if id else None


def data_to_user(data):
    """
    transfer data to user object
    :param data:
    :return:
    """
    alias = data.get('unique_id') or data.get('short_id')
    id = data.get('uid')
    name = data.get('nickname')
    sign = data.get('signature')
    avatar = first(data.get('avatar_larger', {}).get('url_list', []))
    gender = data.get('gender')
    birthday = data.get('birthday')
    create_time = parse_datetime(data.get('create_time'))
    verify = bool(data.get('custom_verify').strip())
    verify_info = data.get('custom_verify').strip()
    return User(
        id=id,
        alias=alias,
        name=name,
        sign=sign,
        avatar=avatar,
        gender=gender,
        verify=verify,
        verify_info=verify_info,
        create_time=create_time,
        birthday=birthday
    ) if id else None


def data_to_address(data):
    id = data.get('poi_id')
    address_info = data.get('address_info', {})
    province = address_info.get('province')
    city = address_info.get('city')
    district = address_info.get('district')
    full = address_info.get('simple_addr')
    address = address_info.get('address')
    postal_code = data.get('type_code')
    longitude = data.get('longitude')
    latitude = data.get('latitude')
    place = data.get('poi_name')
    return Address(
        id=id,
        province=province,
        city=city,
        district=district,
        address=address,
        full=full,
        postal_code=postal_code,
        longitude=longitude,
        latitude=latitude,
        place=place
    ) if id else None



from tqdm import tqdm
import asyncio
import math


class Downloader(object):
    
    def __init__(self, handlers=[], batch=10):
        """
        init attributes
        :param handlers:
        """
        self.handlers = handlers
        self.batch = batch
    
    def add_handler(self, handler):
        """
        add one handler
        :param handler: handler object
        :return:
        """
        self.handlers.append(handler)
    
    def set_handlers(self, handlers):
        """
        set handlers
        :param handlers:
        :return:
        """
        self.handlers = handlers
    
    def get_handlers(self):
        """
        get handlers of downloader
        :return:
        """
        return self.handlers
    
    def update_progress(self, _):
        """
        update progress bar
        :return:
        """
        self.bar.update(1)
    
    async def process_item(self, obj):
        """
        process item
        :param obj: single obj
        :return:
        """
        raise NotImplementedError
    
    def process_items(self, objs):
        """
        process items
        :param objs: objs
        :return:
        """
        # define progress bar
        with tqdm(total=len(objs)) as self.bar:
            # init event loop
            loop = asyncio.get_event_loop()
            # get num of batches
            total_step = int(math.ceil(len(objs) / self.batch))
            # for every batch
            for step in range(total_step):
                start, end = step * self.batch, (step + 1) * self.batch
                print('Downloading %d-%d of files' % (start + 1, end))
                # get batch of objs
                objs_batch = objs[start: end]
                # define tasks and run loop
                tasks = [asyncio.ensure_future(self.process_item(obj)) for obj in objs_batch]
                for task in tasks:
                    task.add_done_callback(self.update_progress)
                loop.run_until_complete(asyncio.wait(tasks))
    
    def download(self, data):
        """
        download video or video lists
        :param data:
        :return:
        """
        data = data if isinstance(data, list) else [data]
        self.process_items(data)



class Address():
    
    def __init__(self, **kwargs):
        self.id = kwargs.get('id')
        self.province = kwargs.get('province')
        self.city = kwargs.get('city')
        self.district = kwargs.get('district')
        self.full = kwargs.get('full')
        self.address = kwargs.get('address')
        self.place = kwargs.get('place')
        self.postal_code = kwargs.get('postal_code')
        self.longitude = kwargs.get('longitude')
        self.latitude = kwargs.get('latitude')
    
    def __repr__(self):
        """
        address to str
        :return:
        """
        return '<Address: <%s, %s>>' % (self.id, self.place)



from os.path import join, exists
from os import makedirs
from douyin.handlers import Handler
from douyin.utils.type import mime_to_ext
import aiohttp


class FileHandler(Handler):
    
    def __init__(self, folder):
        """
        init save folder
        :param folder:
        """
        super().__init__()
        self.folder = folder
        if not exists(self.folder):
            makedirs(self.folder)
    
    async def process(self, url, name, **kwargs):
        """
        download to file
        :param url: resource url
        :param name: save name
        :param kwargs:
        :return:
        """
        kwargs.update({'ssl': False})
        async with aiohttp.ClientSession() as session:
            async with session.get(url, **kwargs) as response:
                if response.status == 200:
                    extension = mime_to_ext(response.headers.get('Content-Type'))
                    full_path = join(self.folder, '%s.%s' % (name, extension))
                    with open(full_path, 'wb') as f:
                        f.write(await response.content.read())
                    print('Downloaded file to', full_path)
                else:
                    print('Cannot download %s, response status %s' % (name, response.status))



from douyin.structures.hot import *
from douyin.structures.music import Music
from douyin.structures.user import User
from douyin.structures.video import Video
from douyin.structures.address import Address


import collections

HotSearch = collections.namedtuple('HotSearch', ['active_time', 'data'])
HotVideo = collections.namedtuple('HotVideo', ['active_time', 'data'])
HotEnergy = collections.namedtuple('HotVideo', ['active_time', 'data'])
HotMusic = collections.namedtuple('HotMusic', ['active_time', 'data'])


class Music(object):
    
    def __init__(self, **kwargs):
        """
        music init args
        :param kwargs:
        """
        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.cover_url = kwargs.get('cover_url')
        self.play_url = kwargs.get('play_url')
        self.owner_id = kwargs.get('owner_id')
        self.owner_name = kwargs.get('owner_name')
        self.hot_count = kwargs.get('hot_count')
    
    def __repr__(self):
        """
        music to str
        :return:
        """
        return '<Music: <%s, %s>>' % (self.id, self.name)
